<?php

namespace Nest\Http;

use Nest\Http\Exceptions\HttpException;
use Nest\Http\Exceptions\InternalErrorException;
use Nyholm\Psr7\Response;
use Override;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Throwable;
use Whoops\Handler\PrettyPageHandler;
use Whoops\Run as Whoops;

/**
 * Default errors handler middleware.
 */
class ErrorsMiddleware implements MiddlewareInterface
{
	/**
	 * @param HttpConfiguration $configuration HTTP service configuration instance.
	 */
	public function __construct(protected HttpConfiguration $configuration)
	{
	}

	/**
	 * @inheritDoc
	 */
	#[Override] public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
	{
		try
		{
			return $handler->handle($request);
		}
		catch (Throwable $exception)
		{ // Handle the exception.
			if (!($exception instanceof HttpException))
			{
				if ($this->configuration->getDebug())
				{ // In debug mode, show details about the thrown exception.
					$whoops = new Whoops();
					$whoops->writeToOutput(false);
					$whoops->allowQuit(false);
					$whoops->pushHandler(new PrettyPageHandler());
					$body = $whoops->handleException($exception);

					// Something bad happened, return an internal error response.
					return new Response(500, body: $body);
				}

				// Otherwise, show an internal error exception.
				$exception = new InternalErrorException(previous: $exception);
			}

			// Should always be an HTTP exception at this point.
			$renderer = new ($this->configuration->getExceptionRenderer())($exception);
			return new Response($renderer->getStatusCode(), body: $renderer->getBody());
		}
	}
}
