<?php

namespace Nest\Http;

use Nest\Http\Exceptions\HttpException;
use Nest\Http\Renderer\PhpRenderer;

/**
 * Render an HTTP exception.
 */
class ExceptionRenderer
{
	/**
	 * @param HttpException $exception The exception to render.
	 */
	public function __construct(protected HttpException $exception)
	{}

	/**
	 * Return the status code for the exception.
	 * @return int Exception status code.
	 */
	public function getStatusCode(): int
	{
		return $this->exception->getStatusCode();
	}

	/**
	 * Return the body of the HTTP exception.
	 * @return string HTTP exception body.
	 */
	public function getBody(): string
	{
		// Render the default exception renderer.
		return (new PhpRenderer(__DIR__."/views/exception.php"))->render((object) [
			"exception" => $this->exception,
		]);
	}
}
