<?php

namespace Nest\Http\Renderer;

use Nest\Application;
use Nest\Http\Exceptions\Renderer\ViewNotFoundException;

/**
 * Plain PHP body renderer.
 */
class PhpRenderer implements Renderer
{
	/**
	 * Resolved view path.
	 * @var string
	 */
	protected string $viewPath;

	/**
	 * @param string $viewName Name of the view to use when rendering.
	 * @throws ViewNotFoundException
	 */
	public function __construct(protected string $viewName)
	{
		// Try to resolve view path.
		$this->viewPath = Application::get()->http()->resolveView($viewName);
	}

	/**
	 * @inheritDoc
	 */
	public function render(object $arguments = new \stdClass()): string
	{
		// Extract render arguments.
		extract((array) $arguments);

		// Start rendering while buffering the output.
		ob_start();
		include $this->viewPath;

		// Return buffered body.
		return ob_get_clean();
	}
}
