<?php

namespace Nest\Http\Router;

use Nest\Http\Exceptions\NotFoundException;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

/**
 * Router HTTP request handler.
 */
class Router
{
	/**
	 * Define a new router request handler from the given routes.
	 * @param Routes $routes Routes instance.
	 */
	public function __construct(protected Routes $routes)
	{
		// Define routes.
		$this->routes->define();
	}

	/**
	 * Handle the request with the right route, if there is one.
	 * @param ServerRequestInterface $request The request to handle.
	 * @return ResponseInterface The request response.
	 * @throws NotFoundException
	 */
	public function __invoke(ServerRequestInterface $request): ResponseInterface
	{
		$route = $this->routes->match($request);
		return $route();
	}
}
